@@ -243,10 +243,11 @@ class awaitable_predicate:
 
     """
 
-    def __init__(self, predicate: typing.Callable[[], bool], condition: asyncio.Condition | None = None):
+    def __init__(self, predicate: typing.Callable[[], bool], condition: asyncio.Condition | None = None, timeout=None):
         self.condition = condition or asyncio.Condition()
         self.predicate = predicate
         self.waiting = None
+        self.timeout = timeout
 
     async def _waiter(self):
         async with self.condition:
@@ -254,9 +255,9 @@ class awaitable_predicate:
 
     def __await__(self):
         if self.waiting is None:
-            self.waiting = self._waiter()
+            self.waiting = asyncio.create_task(self._waiter())
 
-        return self.waiting.__await__()
+        return asyncio.wait_for(self.waiting, timeout=self.timeout).__await__()
 
     def __bool__(self):
         return self.predicate()
\ No newline at end of file
